#!/usr/bin/env python3
from multiprocessing import Pool
import os, json, time, getopt, sys

# binary template for solvers
BINARY_PATH = './solvers/{args[tp]}/train'

# data path template
DATA_DIR = './datasets/{}'

#run_cmd = PRE+"/lr/{args[tp]}/train -s {args[s]} -c {args[c]} {args[fpath]} > {log}"
run_cmd = BINARY_PATH + " -s {args[s]} -c {args[c]} -e {args[e]} {args[fpath]} > {log}"
run_multi_cmd = BINARY_PATH + " -s {args[s]} -n 8 -c {args[c]} -e {args[e]} {args[fpath]} > {log}"

# use best_c or not
USE_BEST = True
if USE_BEST:
    cs = [1, 100]
else:
    cs = [1, 10, 100, 1000]
if USE_BEST:
    LOG_P = './log_{}_best/'
else:
    LOG_P = './log_{}/'
LOG_DIR = LOG_P + '{args[fname]}.{args[tp]}.{args[c_name]}.{args[e]}.log'

# eps options

eps = [1e-10]
# set eps exception for some data sets
eps_exception = {
        'kdda': 1e-6,
        'kddb': 1e-5,
        'kdd12.svm': 1e-5,
        'criteo.trva': 1e-5,
        }
# but solver in eps_ref will use eps
eps_ref = ["origin"]

datasets = [
  "news20.binary",
#  "covtype",
#  "rcv1_train.binary",
#  "w8a",
#  "real-sim",
#  "url_combined",
#  "HIGGS",
#  "yahoojp",
#  "yahookr",
#  "kdda",
#  "kddb",
#  "kdd12.svm",
#  "criteo.trva",
#  "webspam_wc_normalized_trigram.svm"
 ]

if len(datasets) == 0:
    print('Please select some data for experiments !!')
    exit(1)

tps = ["quadada_p",
#  "quadada",
#  "resada",
#  "resada_p",
#  "rescons",
#  "rescons_p",
#  "tr_rescons",
#  "tr_rescons_p",
]

best_c1 = json.load(open('best_c1.json'))
best_c2 = json.load(open('best_c2.json'))

best_c = {'lr': {}, 'l2': {}}
best_c['lr'].update(best_c1['lr'])
best_c['lr'].update(best_c2['lr'])
best_c['l2'].update(best_c1['l2'])
best_c['l2'].update(best_c2['l2'])

pool_num = 1
run_opt = False
fun_type = 'lr'

def usage():
    print('./run.py [-r] [-n thread_num] [-f funtion_type] [-e experiment_number]')

def opt_parse(opts):
    global cs
    for opt in opts:
        if opt[0] == '-r':
            global run_opt
            run_opt = True
        elif opt[0] == '-n':
            if opt[1].isdigit():
                global pool_num
                pool_num = int(opt[1])
            else:
                print('Not an integer !!')
                exit(2)
        elif opt[0] == '-f':
            if opt[1] in ('l2', 'lr'):
                global fun_type
                fun_type = opt[1]
            else:
                print('Function type should be lr or l2 !!')
                exit(2)

def init():
    try:
        opts, args = getopt.getopt(sys.argv[1:],"rn:f:e:")
    except getopt.GetoptError as err:
        print(str(err))
        # print usage
        usage()
        exit(2)
    opt_parse(opts)
    print(fun_type, pool_num, run_opt)

def exec_cmd(cmd, idx=-1, run=False):
    print('RUN #{:2}  {}'.format(idx,cmd))
    if run: os.system(cmd)
    print('Done#{:2}  {}'.format(idx,cmd))

def run_data(args):
    fpath = DATA_DIR.format(args['fname'])
    args['fpath'] = fpath
    #if  args['fname'] == "criteo.trva":
    #    args['fpath'] = "/tmp2/infwinston/criteo.trva"
    binary_path = os.path.realpath(BINARY_PATH.format(args=args))
    if not os.path.exists(binary_path):
        print('No such bianry !!')
        return
    if not os.path.exists(fpath):
        print(fpath,'not exists !!')
        return

    log_p = LOG_P.format(fun_type)
    if not os.path.exists(log_p):
         print('create dir', log_p)
         os.system('mkdir -p {}'.format(log_p))

    log_name = LOG_DIR.format(fun_type, args=args)
    
    if args['tp'][-2:] == '_m':
        cmd = run_multi_cmd.format(args=args, log=log_name)
    else:
        cmd = run_cmd.format(args=args, log=log_name)
    exec_cmd(cmd, idx=args['idx'], run=run_opt)

if __name__ == '__main__':
    init()
    if fun_type == 'lr':
        arg_tmp = {'tp':'line', 's':0, 'c':1, 'e': eps}
    else:
        arg_tmp = {'tp':'line', 's':2, 'c':1, 'e': eps}
    print(fun_type, pool_num, run_opt)
    
    print(len(tps)*len(cs)*len(datasets))
    count = 0
    p =  Pool(pool_num)
    for data in datasets:
        args = []
        for c in cs:
            for e in eps:
                for tp in tps:
                    count += 1
                    arg = dict(arg_tmp)
                    arg['tp'] = tp
                    arg['c_name'] = c
                    if USE_BEST:
                        arg['c'] = best_c[fun_type][data] * c
                    else:
                        arg['c'] = c
                    arg['fname'] = data
                    arg['e'] = e
                    if arg['fname'] in eps_exception and tp not in eps_ref:
                        arg['e'] = eps_exception[arg['fname']]
                    arg['idx'] = count
                    args.append(arg)
        #print(args)
        p.map(run_data, args)
